@@ -2,40 +2,41 @@
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-
-use self::errors::*;
 use self::errors::ErrorKind::*;
-use {SubtitleEntry, SubtitleFile};
+use self::errors::*;
+use crate::{SubtitleEntry, SubtitleFileInterface};
 
+use crate::errors::Result as SubtitleParserResult;
+use crate::formats::common::*;
 use combine::char::{char, string};
 use combine::combinator::{eof, parser as p, skip_many};
 use combine::primitives::Parser;
-use errors::Result as SubtitleParserResult;
-use formats::common::*;
+
+use failure::ResultExt;
 
 use itertools::Itertools;
 
+use crate::timetypes::{TimePoint, TimeSpan};
 use std::iter::once;
-use timetypes::{TimePoint, TimeSpan};
 
-/// `.srt`-parser-specific errors
+type Result<T> = std::result::Result<T, Error>;
+
+/// Errors specific to `.srt`-parsing.
 #[allow(missing_docs)]
 pub mod errors {
-    // see https://docs.rs/error-chain/0.8.1/error_chain/
-    // this error type might be overkill, but that way it stays consistent with
-    // the other parsers
-    error_chain! {
-        errors {
-            ExpectedIndexLine(line: String) {
-                display("expected SubRip index line, found '{}'", line)
-            }
-            ExpectedTimestampLine(line: String) {
-                display("expected SubRip timespan line, found '{}'", line)
-            }
-            ErrorAtLine(line_num: usize) {
-                display("parse error at line `{}`", line_num)
-            }
-        }
+
+    define_error!(Error, ErrorKind);
+
+    #[derive(PartialEq, Debug, Fail)]
+    pub enum ErrorKind {
+        #[fail(display = "expected SubRip index line, found '{}'", line)]
+        ExpectedIndexLine { line: String },
+
+        #[fail(display = "expected SubRip timespan line, found '{}'", line)]
+        ExpectedTimestampLine { line: String },
+
+        #[fail(display = "parse error at line `{}`", line_num)]
+        ErrorAtLine { line_num: usize },
     }
 }
 
@@ -76,11 +77,7 @@ struct SrtLine {
 impl SrtFile {
     /// Parse a `.srt` subtitle string to `SrtFile`.
     pub fn parse(s: &str) -> SubtitleParserResult<SrtFile> {
-        let file_opt = Self::parse_file(s);
-        match file_opt {
-            Ok(file) => Ok(file),
-            Err(err) => Err(err.into()),
-        }
+        Ok(Self::parse_file(s).with_context(|_| crate::ErrorKind::ParsingError)?)
     }
 }
 
@@ -131,15 +128,14 @@ impl SrtFile {
 
     /// Matches a line with a single index.
     fn parse_index_line(line_num: usize, s: &str) -> Result<i64> {
-        s.trim()
-         .parse::<i64>()
-         .chain_err(|| ExpectedIndexLine(s.to_string()))
-         .chain_err(|| ErrorAtLine(line_num))
+        Ok(s.trim()
+            .parse::<i64>()
+            .with_context(|_| ExpectedIndexLine { line: s.to_string() })
+            .with_context(|_| ErrorAtLine { line_num })?)
     }
 
     /// Matches a `SubRip` timespan like "00:24:45,670 --> 00:24:45,680".
     fn parse_timespan_line(line_num: usize, line: &str) -> Result<TimeSpan> {
-
         // Matches a `SubRip` timestamp like "00:24:45,670"
         let timestamp = |s| {
             (
@@ -151,11 +147,11 @@ impl SrtFile {
                 char(','),
                 p(number_i64),
             )
-            .map(|t| TimePoint::from_components(t.0, t.2, t.4, t.6))
-            .parse_stream(s)
+                .map(|t| TimePoint::from_components(t.0, t.2, t.4, t.6))
+                .parse_stream(s)
         };
 
-        (
+        let result = (
             skip_many(ws()),
             p(&timestamp),
             skip_many(ws()),
@@ -165,22 +161,23 @@ impl SrtFile {
             skip_many(ws()),
             eof(),
         )
-        .map(|t| TimeSpan::new(t.1, t.5))
-        .parse(line)
-        .map(|x| x.0)
-        .map_err(|_| Error::from(ExpectedTimestampLine(line.to_string())))
-        .chain_err(|| ErrorAtLine(line_num))
+            .map(|t| TimeSpan::new(t.1, t.5))
+            .parse(line)
+            .map(|x| x.0)
+            .map_err(|_| Error::from(ExpectedTimestampLine { line: line.to_string() }))
+            .with_context(|_| ErrorAtLine { line_num })?;
+
+        Ok(result)
     }
 }
 
-impl SubtitleFile for SrtFile {
+impl SubtitleFileInterface for SrtFile {
     fn get_subtitle_entries(&self) -> SubtitleParserResult<Vec<SubtitleEntry>> {
-        let timings = self.v
-                          .iter()
-                          .map(|line| {
-            SubtitleEntry::new(line.timespan, line.texts.iter().join("\n"))
-        })
-                          .collect();
+        let timings = self
+            .v
+            .iter()
+            .map(|line| SubtitleEntry::new(line.timespan, line.texts.iter().join("\n")))
+            .collect();
 
         Ok(timings)
     }
@@ -199,16 +196,8 @@ impl SubtitleFile for SrtFile {
     }
 
     fn to_data(&self) -> SubtitleParserResult<Vec<u8>> {
-        let timepoint_to_str = |t: TimePoint| -> String {
-
-            format!(
-                "{:02}:{:02}:{:02},{:03}",
-                t.hours(),
-                t.mins_comp(),
-                t.secs_comp(),
-                t.msecs_comp()
-            )
-        };
+        let timepoint_to_str =
+            |t: TimePoint| -> String { format!("{:02}:{:02}:{:02},{:03}", t.hours(), t.mins_comp(), t.secs_comp(), t.msecs_comp()) };
         let line_to_str = |line: &SrtLine| -> String {
             format!(
                 "{}\n{} --> {}\n{}\n\n",
@@ -219,29 +208,22 @@ impl SubtitleFile for SrtFile {
             )
         };
 
-        Ok(
-            self.v
-                .iter()
-                .map(line_to_str)
-                .collect::<String>()
-                .into_bytes(),
-        )
+        Ok(self.v.iter().map(line_to_str).collect::<String>().into_bytes())
     }
 }
 
 impl SrtFile {
     /// Creates .srt file from scratch.
     pub fn create(v: Vec<(TimeSpan, String)>) -> SubtitleParserResult<SrtFile> {
-        let file_parts = v.into_iter()
-                          .enumerate()
-                          .map(|(i, (ts, text))| {
-            SrtLine {
+        let file_parts = v
+            .into_iter()
+            .enumerate()
+            .map(|(i, (ts, text))| SrtLine {
                 index: i as i64 + 1,
                 timespan: ts,
                 texts: text.lines().map(str::to_string).collect(),
-            }
-        })
-                          .collect();
+            })
+            .collect();
 
         Ok(SrtFile { v: file_parts })
     }
@@ -251,17 +233,17 @@ impl SrtFile {
 mod tests {
     #[test]
     fn create_srt_test() {
-        use timetypes::{TimePoint, TimeSpan};
-        use SubtitleFile;
+        use crate::timetypes::{TimePoint, TimeSpan};
+        use crate::SubtitleFileInterface;
 
         let lines = vec![
             (
                 TimeSpan::new(TimePoint::from_msecs(1500), TimePoint::from_msecs(3700)),
-                "line1".to_string()
+                "line1".to_string(),
             ),
             (
                 TimeSpan::new(TimePoint::from_msecs(4500), TimePoint::from_msecs(8700)),
-                "line2".to_string()
+                "line2".to_string(),
             ),
         ];
         let file = super::SrtFile::create(lines).unwrap();
